########################### PSQF 6243 Example 5 using R #####################################

# Set width of output and number of digits to be printed
options("width"=120, "digits"=8)
# Turn off stars for different alpha levels
options(show.signif.stars=FALSE)

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sorting

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("margins")) install.packages("margins")
library(margins) # To get predicted values like Stata does

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files


#############################################################################################
#####              BEGIN DATA MANIPULATION OF HOFFMAN CHAPTER 2 DATA                    #####
#############################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22SP_PSQF6243/PSQF6243_Example5/"
filename = "SAS_Chapter2.sas7bdat"
setwd(dir=filesave)

# Import chapter 2 SAS data using haven package
Example5 = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame without labels to use for analysis
Example5 = as.data.frame(Example5)

# Center quantitative predictors near their means
Example5$age85 = Example5$age-85
Example5$grip9 = Example5$grip-9

# Create 2 indicator-dummy-coded binary indicator predictors for 3 dementia groups
Example5$demNF = NA; Example5$demNC = NA  # Create 2 new empty variables
# Recode if demgroup=none
Example5$demNF[which(Example5$demgroup==1)]=0 
Example5$demNC[which(Example5$demgroup==1)]=0
# Recode if demgroup=future
Example5$demNF[which(Example5$demgroup==2)]=1 
Example5$demNC[which(Example5$demgroup==2)]=0
# Recode if demgroup=current
Example5$demNF[which(Example5$demgroup==3)]=0 
Example5$demNC[which(Example5$demgroup==3)]=1

# Label all variables as comments only (not actually added to data)
#age85=     "age85: Age in Years (0=85)"
#grip9=     "grip9: Grip Strength in Pounds (0=9)"
#sexMW=     "sexMW: Sex (0=M, 1=W)"
#demNF=     "demNF: Dementia Predictor for None=0 vs Future=1"
#demNC=     "demNC: Dementia Predictor for None=0 vs Current=1"
#cognition= "cognition: Cognition Outcome"
#demgroup=  "demgroup: Dementia Group 1N 2F 3C"

# Filter to only cases complete on all variables to be used below
Example5 = Example5[complete.cases(Example5[ , 
            c("cognition","age","grip","sexMW","demgroup")]),]

# Open output text file
txtStart(file=paste0(filesave,"Example5_R_Output.txt"))


#############################################################################################
#####                         BEGIN GENERAL LINEAR MODELS                               #####
#############################################################################################

print("R Combined Main-Effects-Only Model Predicting Cognition")
ModelMain = lm(data=Example5, formula=cognition~1+age85+grip9+sexMW+demNF+demNC)
anova(ModelMain)    # anova to print residual variance
summary(ModelMain)  # print fixed effects solution
confint(ModelMain, level=.95) # confint for level% CI for fixed effects

print("Get DFnum=2 F-test for demgroup") # Omnibus group main effect
mainFdem = glht(model=ModelMain, linfct=c("demNF=0","demNC=0"))
summary(mainFdem, test=Ftest()) # ask for joint hypothesis test instead of separate

print("Get missing demgroup difference: Future vs Current")
DemMain = summary(glht(model=ModelMain, linfct=rbind(c(0,0,0,0,-1,1))),
                       test=adjusted("none")) # Beta5-Beta4
print(DemMain); confint(DemMain, level=.95, calpha=univariate_calpha())

print("Pred cognition outcomes holding sexMW=men, demNF=none, and demNC=none")
print("The LONGER WAY -- writing out separate glht statement per prediction")
PredMain = glht(model=ModelMain, linfct=rbind(
  "Yhat for Age=80 Grip=6"  = c(1,-5,-3, 0,0,0),  # in order of fixed effects
  "Yhat for Age=80 Grip=9"  = c(1,-5, 0, 0,0,0),
  "Yhat for Age=80 Grip=12" = c(1,-5, 3, 0,0,0),
  "Yhat for Age=85 Grip=6"  = c(1, 0,-3, 0,0,0),
  "Yhat for Age=85 Grip=9"  = c(1, 0, 0, 0,0,0),
  "Yhat for Age=85 Grip=12" = c(1, 0, 3, 0,0,0),
  "Yhat for Age=90 Grip=6"  = c(1, 5,-3, 0,0,0),
  "Yhat for Age=90 Grip=9"  = c(1, 5, 0, 0,0,0),
  "Yhat for Age=90 Grip=12" = c(1, 5, 3, 0,0,0)))
summary(PredMain, test=adjusted("none"))
confint(PredMain, level=.95, calpha=univariate_calpha()) # unadjusted CIs

print("The SHORTER WAY -- one prediction replaces 9 glht statements")
print("Provides predicted outcomes from min,max,by=increment of predictors")
PredMainP = prediction(model=ModelMain, type="response", at=list(
             sexMW=0, demNF=0, demNC=0, grip9=seq(-3,3,by=3), age85=seq(-5,5,by=5)))
PlotMain = summary(PredMainP); PlotMain # Save predictions for plotting

# Make and save plot of Predicted Outcomes -- now working correctly
png(file = "R Main-Effects-Only GLM Plot.png")  # open file
plot(y=PlotMain$Prediction, x=(PlotMain$`at(grip9)`+9), type="n",  # n= no points
		 ylim=c(15,45), xlim=c(6,12), xlab="Grip Strength",ylab="Predicted Cognition")
lines(x=(PlotMain$`at(grip9)`+9)[1:3], y=PlotMain$Prediction[1:3], type="l", col="blue1")
lines(x=(PlotMain$`at(grip9)`+9)[4:6], y=PlotMain$Prediction[4:6], type="l", col="red1")
lines(x=(PlotMain$`at(grip9)`+9)[7:9], y=PlotMain$Prediction[7:9], type="l", col="green1")
legend(x=6, y=45, legend=c("Age 80","Age 85","Age 90"), col=1:3, lty=1) # lty=linetype
dev.off()  # close file

txtComment(" ") # insert blank space in output  



print("R GLM Adding 2 Interactions -- Sex by Age, Sex by Grip")
ModelInt2 = lm(data=Example5, formula=cognition~1+age85+grip9+sexMW+demNF+demNC 
                                                 +age85:sexMW +grip9:sexMW)
anova(ModelInt2)    # anova to print residual variance
summary(ModelInt2)  # print fixed effects solution
confint(ModelInt2, level=.95) # confint for level% CI for fixed effects

print("Get DFnum=2 F-test for demgroup") # Omnibus group main effect
Int2Fdem = glht(model=ModelInt2,linfct=c("demNF=0","demNC=0"))
summary(Int2Fdem, test=Ftest()) # ask for joint hypothesis test instead of separate

print("Get DFnum=2 F-test for new interactions")
Int2FInt = glht(model=ModelInt2, linfct=c("age85:sexMW=0","grip9:sexMW=0"))
summary(Int2FInt, test=Ftest()) # ask for joint hypothesis test instead of separate

print("Missing dem diff; simple slopes of age by sex, simple slopes of sex by age")
SlopesInt2 = glht(model=ModelInt2, linfct=rbind(
  "Future vs Current Diff" = c(0,0,0,0,-1,1,0,0),  # Beta5-Beta4
  "Age Slope for Men"      = c(0,1,0,0, 0,0,0,0),  # in order of fixed effects
  "Age Slope for Women"    = c(0,1,0,0, 0,0,1,0),
  "Grip Slope for Men"     = c(0,0,1,0, 0,0,0,0),
  "Grip Slope for Women"   = c(0,0,1,0, 0,0,0,1)))
summary(SlopesInt2, test=adjusted("none"))
confint(SlopesInt2, level=.95, calpha=univariate_calpha()) # unadjusted CIs

print("Pred cognition outcomes holding demNF=none, and demNC=none")
print("The SHORTER WAY -- one prediction replaces 8 separate glht statements")
print("Provides predicted outcomes from min,max,by=increment of predictors")
PredInt2 = prediction(model=ModelInt2, type="response", at=list(
           demNF=0, demNC=0, grip9=seq(-3,3,by=6), age85=seq(-5,5,by=10), sexMW=0:1))
PlotInt2 = summary(PredInt2); PlotInt2 # Save predictions for plotting

# Make and save plots of Predicted Outcomes -- now working correctly
png(file = "R Sex by Grip=x GLM Plot.png")  # open file
PlotInt2age80=PlotInt2[which(PlotInt2$`at(age85)`==-5),] # Subset to age=80
plot(y=PlotInt2age80$Prediction, x=(PlotInt2age80$`at(grip9)`+9), type="n",  # n= no dots
     ylim=c(15,45), xlim=c(6,12), xlab="Grip Strength",ylab="Predicted Cognition")
lines(x=(PlotInt2age80$`at(grip9)`+9)[1:2], y=PlotInt2age80$Prediction[1:2], type="l", col="blue1")
lines(x=(PlotInt2age80$`at(grip9)`+9)[3:4], y=PlotInt2age80$Prediction[3:4], type="l", col="red1")
legend(x=6, y=45, legend=c("Men","Women"), col=1:2, lty=1) # lty=linetype
dev.off()  # close file
png(file = "R Sex by Age=x GLM Plot.png")  # open file
PlotInt2grip6=PlotInt2[which(PlotInt2$`at(grip9)`==-3),] # Subset to grip=6
plot(y=PlotInt2grip6$Prediction, x=(PlotInt2grip6$`at(grip9)`+9), type="n",  # n= no dots
     ylim=c(15,45), xlim=c(80,90), xlab="Years of Age",ylab="Predicted Cognition")
lines(x=(PlotInt2grip6$`at(age85)`+85)[1:2], y=PlotInt2grip6$Prediction[1:2], type="l", col="blue1")
lines(x=(PlotInt2grip6$`at(age85)`+85)[3:4], y=PlotInt2grip6$Prediction[3:4], type="l", col="red1")
legend(x=80, y=45, legend=c("Men","Women"), col=1:2, lty=1) # lty=linetype
dev.off()  # close file

print("Geting semipartial eta-squares as given by SAS CONTRAST")
print("Save total sums of squares from empty model")
ModelEmpty = lm(data=Example5, formula=cognition~1)
SStotal=anova(ModelEmpty)$`Sum Sq`

print("R Semi-Partial R2 for DemGroup via SS for Reduced Model")
ModelNoDem = lm(data=Example5, formula=cognition~1+age85+grip9+sexMW+age85:sexMW+grip9:sexMW)
SSDem=anova(ModelNoDem, ModelInt2); SSDem$`Sum of Sq`/SStotal

print("R Semi-Partial R2 for 2 Sex Interactions via SS for Reduced Model")
ModelNoInt2 = lm(data=Example5, formula=cognition~1+age85+grip9+sexMW+demNF+demNC)
SSInt2=anova(ModelNoInt2, ModelInt2); SSInt2$`Sum of Sq`/SStotal

txtComment(" ") # insert blank space in output  


print("R Eq 2.9: GLM Adding Age by Grip Strength Interaction")
ModelAgeGrip = lm(data=Example5, formula=cognition~1+age85+grip9+sexMW+demNF+demNC 
                                                    +age85:grip9)
anova(ModelAgeGrip)    # anova to print residual variance
summary(ModelAgeGrip)  # print fixed effects solution
confint(ModelAgeGrip, level=.95) # confint for level% CI for fixed effects

print("Get DFnum=2 F-test for demgroup") # Omnibus group main effect
AgeGripFdem = glht(model=ModelAgeGrip,linfct=c("demNF=0","demNC=0"))
summary(AgeGripFdem, test=Ftest()) # ask for joint hypothesis test instead of separate

print("Get DFnum=3 F-test for age, grip, and age*grip")
AgeGripF = glht(model=ModelAgeGrip, linfct=c("age85=0","grip9=0","age85:grip9=0"))
summary(AgeGripF, test=Ftest()) # ask for joint hypothesis test instead of separate

print("Missing dem diff; Simple slopes for age per grip, simple slopes for grip per age")
SlopesAgeGrip = glht(model=ModelAgeGrip, linfct=rbind(
  "Future vs Current Diff" = c(0,0,0,0,-1,1,0),  # Beta5-Beta4
  "Age Slope at Grip = 6"  = c(0,1,0,0,0,0,-3),  # in order of fixed effects
  "Age Slope at Grip = 9"  = c(0,1,0,0,0,0, 0),
  "Age Slope at Grip = 12" = c(0,1,0,0,0,0, 3),
  "Grip Slope at Age = 80" = c(0,0,1,0,0,0,-5),
  "Grip Slope at Age = 85" = c(0,0,1,0,0,0, 0),
  "Grip Slope at Age = 90" = c(0,0,1,0,0,0, 5)))
SaveSlopes = summary(SlopesAgeGrip, test=adjusted("none")); SaveSlopes
confint(SlopesAgeGrip, level=.95, calpha=univariate_calpha()) # unadjusted CIs

print("Pred cognition outcomes holding sexMW=0, demNF=none, and demNC=none")
print("The SHORTER WAY -- one prediction replaces 9 glht statements")
print("Provides predicted outcomes from min,max,by=increment of predictors")
PredInt3 = prediction(model=ModelAgeGrip, type="response", at=list(
            demNF=0, demNC=0, sexMW=0, grip9=seq(-3,3,by=3), age85=seq(-5,5,by=5)))
PlotInt3 = summary(PredInt3); PlotInt3 # Save predictions for plotting

# Make and save plots -- now working correctly
png(file = "R Age by Grip=x GLM Plot.png")  # open file
PlotInt3 = sort_asc(data=PlotInt3,`at(age85)`) # 3 rows per age
plot(y=PlotInt3$Prediction, x=(PlotInt3$`at(grip9)`+9), type="n",  # n= no points
     ylim=c(15,45), xlim=c(6,12), xlab="Grip Strength",ylab="Predicted Cognition")
lines(x=(PlotInt3$`at(grip9)`+9)[1:3], y=PlotInt3$Prediction[1:3], type="l", col="blue1")
lines(x=(PlotInt3$`at(grip9)`+9)[4:6], y=PlotInt3$Prediction[4:6], type="l", col="red1")
lines(x=(PlotInt3$`at(grip9)`+9)[7:9], y=PlotInt3$Prediction[7:9], type="l", col="green1")
legend(x=6, y=45, legend=c("Age 80","Age 85","Age 90"), col=1:3, lty=1) # lty=linetype
dev.off()  # close file
png(file = "R Grip by Age=x GLM Plot.png")  # open file
PlotInt3 = sort_asc(data=PlotInt3,`at(grip9)`) # 3 rows per grip
plot(y=PlotInt3$Prediction, x=(PlotInt3$`at(age85)`+85), type="n",  # n= no points
     ylim=c(15,45), xlim=c(80,90), xlab="Years of Age",ylab="Predicted Cognition")
lines(x=(PlotInt3$`at(age85)`+85)[1:3], y=PlotInt3$Prediction[1:3], type="l", col="blue1")
lines(x=(PlotInt3$`at(age85)`+85)[4:6], y=PlotInt3$Prediction[4:6], type="l", col="red1")
lines(x=(PlotInt3$`at(age85)`+85)[7:9], y=PlotInt3$Prediction[7:9], type="l", col="green1")
legend(x=80, y=45, legend = c("Grip=6", "Grip=9", "Grip=12"), col=1:3, lty=1) # lty=linetype
dev.off()  # close file

print("R Semi-Partial R2 for DemGroup via SS for Reduced Model")
ModelNoDem3 = lm(data=Example5, formula=cognition~1+age85+grip9+sexMW+age85:grip9)
SSDem=anova(ModelNoDem3, ModelAgeGrip); SSDem$`Sum of Sq`/SStotal

print("R Semi-Partial R2 for Age, Grip, and Age*Grip via SS for Reduced Model")
ModelNoInt3 = lm(data=Example5, formula=cognition~1+sexMW+demNF+demNC)
SSInt3=anova(ModelNoInt3, ModelAgeGrip); SSInt3$`Sum of Sq`/SStotal

print("R code to compute effect sizes from stored model results from fixed effects")
SumModelAgeGrip=summary(ModelAgeGrip)
ModelAllpD=(2*SumModelAgeGrip$coefficients[,"t value"])/sqrt(ModelAgeGrip$df.residual)
ModelAllpR=   SumModelAgeGrip$coefficients[,"t value"]/
         sqrt(SumModelAgeGrip$coefficients[,"t value"]^2+ModelAgeGrip$df.residual)
# Concatenate effect sizes to results table for fixed effects
data.frame(SumModelAgeGrip$coefficients, PartialD=ModelAllpD, PartialR=ModelAllpR)

print("R code to compute effect sizes from stored model results from GLHT")
SlopesPartialD=(2*SaveSlopes$test$tstat)/sqrt(ModelAgeGrip$df.residual)
SlopesPartialR=   SaveSlopes$test$tstat/sqrt(SaveSlopes$test$tstat^2+ModelAgeGrip$df.residual)
# Concatenate effect sizes to results table for mean differences
data.frame(Estimate=SaveSlopes$test$coefficients, SE=SaveSlopes$test$sigma,
           pvalue=SaveSlopes$test$pvalues, 
           PartialD=SlopesPartialD, PartialR=SlopesPartialR)

txtComment(" ") # insert blank space in output  


########## BONUS: SIMPLE SLOPES MORE EASILY AND REGIONS OF SIGNIFICANCE ############

if (!require("reghelper")) install.packages("reghelper")
library(reghelper) # To do efficient simple slopes in GLMs

print("Simple slopes over range of moderator values using reghelper package")
simple_slopes(model=ModelAgeGrip, levels=list(age85=c(-5,0,5,'sstest'),grip9=c(-3,0,3,'sstest')))

if (!require("interactions")) install.packages("interactions")
library(interactions) # To do regions of significance in GLMs

print("Regions of significance using interactions package") # plots broke my computer
ModelAgeGrip = lm(data=Example5, formula=cognition~1+age85+grip9+sexMW+demNF+demNC 
                                                    +age85:grip9)
vcov(ModelAgeGrip) # Asymptotic covariance matrix of fixed effects for regions
johnson_neyman(model=ModelAgeGrip, pred="age85", modx="grip9", digits=3, plot=FALSE) 
johnson_neyman(model=ModelAgeGrip, pred="grip9", modx="age85", digits=3, plot=FALSE)

txtComment(" ") # insert blank space in output  


# Close output text file
txtStop()


